@@ -15,9 +15,10 @@ ShapeMaker::ShapeMaker( List<ShapeItem>* e, double fx, double fy, double ofsx, d
 	factory = fy;
 	offsetx = ofsx;
 	offsety = ofsy;
-	diffx = diffy = 0;
-	lastx = lasty = 0;
-	smoothx = smoothy = 0;
+	diffx = diffy = 0.0;
+	lastx = lasty = 0.0;
+	smoothx = smoothy = 0.0;
+	minx = miny = maxx = maxy = 0.0;
 	have_first = false;
 	
 	fillStyle0 = lineStyle = fillStyle1 = -1;
@@ -95,6 +96,7 @@ void ShapeMaker::curveToR( double _cx, double _cy, double ax, double ay ) {
 
 	diffx += cx + x; diffy += cy + y;
 	
+	
 	CurveTo *segment = new CurveTo;
 	segment->setType(2);
 	segment->setbits( SWFMaxBitsNeeded( true, 4, x, y, cx, cy ) );
@@ -124,93 +126,78 @@ void ShapeMaker::smoothCurveTo( double ax, double ay ) {
 	curveTo( lastx + smoothx, lasty + smoothy, ax, ay );
 }
 
-// cubic to quadratic bezier functions
-// thanks to Robert Penner
-
-Point intersectLines( Point p1, Point p2, Point p3, Point p4 ) {
-	double x1 = p1.x, y1 = p1.y;
-	double x4 = p4.x, y4 = p4.y;
-	double dx1 = p2.x-x1;
-	double dx2 = p3.x-x4;
-	if( dx1 == 0 && dx2 == 0 ) return Point(p1.x + ((p4.x-p1.x)/2), p1.y + ((p4.y-p1.y)/2));
-	double m1 = (p2.y-y1)/dx1;
-	double m2 = (p3.y-y4)/dx2;
-	
-	if( !dx1 ) {
-		return Point( x1, (m2*(x1-x4))+y4 );
-	} else if( !dx2 ) {
-		return Point( x4, (m1*(x4-x1))+y1 );
-	} else if( fabs( m1-m2 ) < .001 ) {
-		return Point( x4, (m1*(x4-x1))+y1 );
-	}
-	double x = ((-m2 * x4) + y4 + (m1 * x1) - y1) / (m1-m2);
-	Point p( x, (m1 * (x-x1)) + y1 );
-	
-/*	
-	if( fabs(p.y) > 100 ) {
-		fprintf(stderr,"p1/2: %f/%f %f/%f %f/%f %f/%f\n", p1.x, p1.y, p2.x, p2.y, p3.x, p3.y, p4.x, p4.y );
-		fprintf(stderr,"S: %f/%f  m1/2: %f/%f\n", p.x, p.y, m1, m2 );
-	}
-*/
-	return p;
-}
-
-Point midLine( const Point& a, const Point& b ) {
-	Point p( ((a.x+b.x)/2), ((a.y+b.y)/2) );
-	return p;
-}
-
-void bezierSplit( const Bezier& b, Bezier* b0, Bezier* b1 ) {
-	Point p01 = midLine( b.p0, b.p1 );
-	Point p12 = midLine( b.p1, b.p2 );
-	Point p23 = midLine( b.p2, b.p3 );
-	Point p02 = midLine( p01, p12 );
-	Point p13 = midLine( p12, p23 );
-	Point p03 = midLine( p02, p13 );
-	
-	b0->set( b.p0, p01, p02, p03 );
-	b1->set( p03, p13, p23, b.p3 );
-}
-
-void ShapeMaker::cubicToRec( const Point& a, const Point& b, const Point& c, const Point& d, double k, int iteration ) {
-	Point s = intersectLines( a, b, c, d );
-	double dx = (a.x+d.x+s.x*4-(b.x+c.x)*3)*.125;
-	double dy = (a.y+d.y+s.y*4-(b.y+c.y)*3)*.125;
-	Bezier bz( a, b, c, d );
-	Bezier b0, b1;
-	if( dx*dx + dy*dy > k && iteration<10 ) {
-//		fprintf(stderr,"[%03i] split: %f\n", iteration, dx*dx + dy*dy);
-		bezierSplit( bz, &b0, &b1 );
-		// recurse
-		iteration++;
-		cubicToRec( a,    b0.p1, b0.p2, b0.p3, k, iteration );
-		//lineTo( b0.p3.x, b0.p3.y );
-		
-		cubicToRec( b1.p0, b1.p1, b1.p2, d,    k, iteration );
-		//lineTo( b1.p1.x, b1.p1.y );
-		//lineTo( b1.p2.x, b1.p2.y );
-		//lineTo( d.x, d.y );
-	} else {
-//		fprintf(stderr,"#### %i %i %i %i\n", (int)s.x, (int)s.y, (int)d.x, (int)d.y );
-		//lineTo( (int)s.x, (int)s.y );
-		//lineTo( (int)d.x, (int)d.y );
-		curveTo( s.x, s.y, d.x, d.y );
-	}
-}
-
 void ShapeMaker::cubicTo( double x1, double y1, double x2, double y2, double ax, double ay ) {
 	Point a(lastx,lasty);
 	Point b(x1,y1);
 	Point c(x2,y2);
 	Point d(ax,ay);
 
-	cubicToRec( a, b, c, d, .01 );
-	
+	Bezier cubic (a, b, c, d);
+
+	double t0, t1;
+	int cInflections = cubic.computeInflections (t0, t1);
+
+	if (cInflections == 0)
+	{
+		cubicTo(cubic);
+	}
+	else if (cInflections == 1)
+	{
+		Bezier cubic0 = cubic.split (t0);
+		cubicTo (cubic0);
+		cubicTo (cubic);
+	}
+	else
+	{
+		Bezier cubic0 = cubic.split (t0);
+		cubicTo (cubic0);
+
+		Bezier cubic1 = cubic.split (1 - (1 - t1) / (1 - t0));
+		cubicTo (cubic1);
+
+		cubicTo (cubic);
+	}
+
 	lastx = ax; lasty = ay;
 	smoothx = ax - x2;
 	smoothy = ay - y2;
 }
 
+void ShapeMaker::cubicTo (const Bezier &cubic)
+{
+	const double sqrt3 = 1.7320508075688772935274463415059;
+	const double precision = 600 * 18 / (60 * factorx * sqrt3);
+
+	// distance between control points of quadratic approximations of either end
+	double D01 = (cubic.p1 - (cubic.c1 - cubic.c0) * 3 - cubic.p0).magnitude () / 2;
+
+	double tMax3 = precision / D01;
+	if (tMax3 >= 1)
+	{
+		curveTo (cubic.quadraticCtrl(), cubic.p1);
+		return;
+	}
+
+	Bezier end(cubic);
+
+	if (tMax3 >= 0.5 * 0.5 * 0.5)
+	{
+		Bezier start (end.split (0.5));
+		curveTo (start.quadraticCtrl (), start.p1);
+	}
+	else
+	{
+		double tMax = pow (tMax3, 1.0/3.0);
+		Bezier start (end.split (tMax));
+		Bezier middle (end.split (1 - tMax / (1 - tMax)));
+
+		curveTo (start.quadraticCtrl (), start.p1);
+		cubicTo (middle);
+	}
+
+	curveTo (end.quadraticCtrl (), end.p1);
+}
+
 void ShapeMaker::cubicToR( double x1, double y1, double x2, double y2, double ax, double ay ) {
 	cubicTo(lastx + x1, lasty + y1,
 	        lastx + x2, lasty + y2,
@@ -230,9 +217,6 @@ void ShapeMaker::close(bool stroke) {
 	// diffx/diffy captures rounding errors. they can accumulate a bit! FIXME
 	
 	if( diffx || diffy ) {
-		/*fprintf(stderr,"WARNING: shape not closed; closing (%f/%f).\n", diffx, diffy);
-		fprintf(stderr,"DEBUG: accumulated rounding error (%f/%f).\n", roundx, roundy);*/
-		
 		if(!stroke) {
 			doSetup( 0, 0, false, -1, -1, 0 );
 		}
@@ -257,6 +241,11 @@ void ShapeMaker::close(bool stroke) {
 }
 
 void ShapeMaker::finish() {
+	ListItem<ShapeItem> *last = edges->last ();
+	if (last && last->data()->isEnd ()) {
+		return;
+	}
+
 	// end shape
 	ShapeSetup *setup = new ShapeSetup;
 	edges->append( setup );
@@ -307,12 +296,12 @@ void ShapeMaker::ellipseSegment( double cx, double cy, double rx, double ry, dou
 	double a2 = theta + dTheta;
 	double f = cos(dTheta / 2);
 
-	Point p1(cos(a1) * rx / f, sin(a1) * ry / f);
-	Point p2(cos(a2) * rx, sin(a2) * ry);
-	p1.rotate(phi);
-	p2.rotate(phi);
+	Point c0(cos(a1) * rx / f, sin(a1) * ry / f);
+	Point c1(cos(a2) * rx, sin(a2) * ry);
+	c0.rotate(phi);
+	c1.rotate(phi);
 
-	curveTo(cx + p1.x, cy + p1.y, cx + p2.x, cy + p2.y);
+	curveTo(cx + c0.x, cy + c0.y, cx + c1.x, cy + c1.y);
 }
 
 void ShapeMaker::ellipse( double cx, double cy, double rx, double ry ) {
