@@ -5,19 +5,23 @@
 #include <string>
 #include <stack>
 #include <map>
-	
+#include "SVGGradient.h"
+#include "SVGStyle.h"
+
 void swft_register();
-	
-// utility functoins for import (in swft_import.cpp)
+
+// utility functions for import (in swft_import.cpp)
 void swft_addFileName( xmlNodePtr node, const char *filename );
 void swft_addData( xmlNodePtr node, char *data, int length );
 
-unsigned char *swft_get_filename( xmlChar *string );
+char *swft_get_filename( const xmlChar *uri, const xmlChar *baseUri );
 
 #define SWFT_NAMESPACE ((const xmlChar*)"http://subsignal.org/swfml/swft")
 
 class swft_ctx {
 public:
+	std::stack<SWF::SVGStyle> styles;
+	std::map<std::string, SWF::SVGGradient*> gradients;
 
 	int last_id;
 	int last_depth;
@@ -57,30 +61,4 @@ public:
 	}
 };
 
-// CSS style stuff
-
-struct CSSColor {
-	CSSColor() {
-			r=g=b=a=0;
-		};
-		
-	unsigned char r, g, b, a;
-};
-
-struct CSSStyle {
-	CSSStyle() {
-		no_fill = no_stroke = false;
-		width = 0;
-	}
-	
-	bool no_fill, no_stroke;
-	CSSColor fill;
-	CSSColor stroke;
-	double width;
-	
-	std::map<std::string,std::string> styles;
-};
-
-void parse_css_simple( const char *style_str, CSSStyle *style );
-
 #endif
